"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.usePagination = void 0;

var _react = _interopRequireDefault(require("react"));

var _getDataPerPage = require("../utils/getDataPerPage");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var usePagination = function usePagination(_ref) {
  var data = _ref.data,
      page = _ref.page,
      _ref$defaultPage = _ref.defaultPage,
      defaultPage = _ref$defaultPage === void 0 ? 1 : _ref$defaultPage,
      _ref$rowsPerPage = _ref.rowsPerPage,
      rowsPerPage = _ref$rowsPerPage === void 0 ? 0 : _ref$rowsPerPage,
      _ref$dataSize = _ref.dataSize,
      dataSizeProp = _ref$dataSize === void 0 ? 0 : _ref$dataSize,
      onChange = _ref.onChange;

  var _React$useRef = _react["default"].useRef(page !== undefined),
      isControlled = _React$useRef.current;

  var _React$useState = _react["default"].useState(defaultPage),
      _React$useState2 = _slicedToArray(_React$useState, 2),
      pageState = _React$useState2[0],
      setPageState = _React$useState2[1];

  var value = isControlled ? page || 1 : pageState;
  var dataSize = data !== undefined ? data.length : dataSizeProp;

  var setValueIfUncontrolled = _react["default"].useCallback(function (newValue) {
    if (!isControlled) {
      setPageState(newValue);
    }
  }, [isControlled]);

  var handleChange = function handleChange(newValue) {
    setValueIfUncontrolled(newValue);

    if (onChange) {
      onChange(newValue);
    }
  };

  var numberOfPages = _react["default"].useMemo(function () {
    return Math.ceil(dataSize / (rowsPerPage || dataSize));
  }, [dataSize, rowsPerPage]);

  var range = _react["default"].useCallback(function () {
    if (isNaN(value) || numberOfPages === 1) {
      return [];
    }

    var input = [];

    if (value > 2) {
      input.push(1);
    }

    if (value === 4) {
      input.push(2);
    }

    if (value > 4) {
      input.push('.');
    }

    if (value > 1) {
      input.push(value - 1);
    }

    input.push(value);

    if (value < numberOfPages) {
      input.push(value + 1);
    }

    if (value === numberOfPages - 3) {
      input.push(numberOfPages - 1);
    }

    if (value < numberOfPages - 3) {
      input.push('.');
    }

    if (value < numberOfPages - 1) {
      input.push(numberOfPages);
    }

    return input;
  }, [value, numberOfPages]);

  var dataPerPage = _react["default"].useMemo(function () {
    return rowsPerPage && data ? (0, _getDataPerPage.getDataPerPage)(data, value, rowsPerPage) : data;
  }, [value, rowsPerPage, data]);

  var lastRowIndex = value * rowsPerPage;
  var firstRowIndex = lastRowIndex - rowsPerPage + 1;
  lastRowIndex = Math.min(dataSize, lastRowIndex);
  var hasPreviousPage = value > 1;
  var hasNextPage = numberOfPages > 1 && numberOfPages - value !== 0;
  return {
    rowsPerPage: rowsPerPage,
    page: value,
    setPage: handleChange,
    dataSize: dataSize,
    firstRowIndex: firstRowIndex,
    lastRowIndex: lastRowIndex,
    hasPreviousPage: hasPreviousPage,
    hasNextPage: hasNextPage,
    range: range,
    numberOfPages: numberOfPages,
    dataPerPage: dataPerPage
  };
};

exports.usePagination = usePagination;