"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useVirtualTable = void 0;

var _react = _interopRequireDefault(require("react"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var EXTRA_ROWS_NUMBER = 8;

var useVirtualTable = function useVirtualTable(data, _ref) {
  var tableWrapRef = _ref.tableWrapRef,
      headerCellRefs = _ref.headerCellRefs;

  var _React$useState = _react["default"].useState(0),
      _React$useState2 = _slicedToArray(_React$useState, 2),
      currentIndex = _React$useState2[0],
      setCurrentIndex = _React$useState2[1];

  var _React$useState3 = _react["default"].useState(1),
      _React$useState4 = _slicedToArray(_React$useState3, 2),
      numberOfVisibleItems = _React$useState4[0],
      setNumberOfVisibleItems = _React$useState4[1];

  var containerHeight = _react["default"].useRef(0);

  var rootRef = _react["default"].useRef();

  var bodyRef = _react["default"].useRef();

  var itemsCount = data.length;

  var displayData = _react["default"].useMemo(function () {
    return data.slice(currentIndex, currentIndex + numberOfVisibleItems);
  }, [data, currentIndex, numberOfVisibleItems]);

  _react["default"].useLayoutEffect(function () {
    if (displayData.length) {
      // calculate container height
      var firstRowNode = bodyRef.current.querySelector('.cb-TableRow');

      if (firstRowNode) {
        var rowHeight = firstRowNode.offsetHeight;
        var newNumberOfVisibleElements = Math.ceil(rootRef.current.offsetHeight / rowHeight) + 3 + EXTRA_ROWS_NUMBER;
        containerHeight.current = rowHeight * itemsCount;

        if (newNumberOfVisibleElements !== numberOfVisibleItems) {
          setNumberOfVisibleItems(newNumberOfVisibleElements);
        }
      }
    }
  }, [displayData]);

  var onScroll = function onScroll() {
    var firstRowNode = bodyRef.current.querySelector('.cb-TableRow');

    if (firstRowNode) {
      var scrollTop = rootRef.current.scrollTop;

      if (scrollTop >= 0) {
        var rowHeight = firstRowNode.offsetHeight;
        var newIndex = Math.max(0, Math.floor((scrollTop - EXTRA_ROWS_NUMBER * rowHeight) / rowHeight));

        if (newIndex !== currentIndex) {
          var newOffset = newIndex * rowHeight;
          tableWrapRef.current.style.transform = "translateY(".concat(newOffset, "px)");

          if (headerCellRefs) {
            var _headerCellRefs$curre;

            (_headerCellRefs$curre = headerCellRefs.current) === null || _headerCellRefs$curre === void 0 ? void 0 : _headerCellRefs$curre.forEach(function (element) {
              element.style.top = "-".concat(newOffset, "px");
            });
          }

          setCurrentIndex(newIndex);
        }
      }
    }
  };

  return {
    rootProps: {
      ref: rootRef,
      onScroll: onScroll
    },
    tableProps: {
      data: displayData
    },
    tableBodyProps: {
      ref: bodyRef,
      currentIndex: currentIndex
    },
    containerHeight: containerHeight.current
  };
};

exports.useVirtualTable = useVirtualTable;